@@ -15,3 +15,6 @@ CONFIG_EMAIL = "email"
 CONFIG_MODEL_PATH = "model_path"
 CONFIG_DWH_HTTP_PATH = "dwh_http_path"
 CONFIG_DWH_ACCESS_TOKEN = "dwh_access_token"
+CONFIG_DBT_REPO = "dbt_repo"
+CONFIG_DBT_PROFILE = "dbt_profile"
+CONFIG_DBT_TARGET = "dbt_target"
