@@ -36,7 +36,7 @@ from metricflow.specs import (
     LinklessIdentifierSpec,
     InstanceSpecSet,
 )
-from metricflow.model.semantics.semantic_containers import DataSourceSemantics
+from metricflow.protocols.semantics import DataSourceSemanticsAccessor
 
 logger = logging.getLogger(__name__)
 
@@ -105,7 +105,7 @@ class NodeEvaluatorForLinkableInstances(Generic[SourceDataSetT]):
 
     def __init__(
         self,
-        data_source_semantics: DataSourceSemantics,
+        data_source_semantics: DataSourceSemanticsAccessor,
         nodes_available_for_joins: Sequence[BaseOutput[SourceDataSetT]],
         node_data_set_resolver: DataflowPlanNodeOutputDataSetResolver[SourceDataSetT],
     ) -> None:
@@ -157,7 +157,12 @@ class NodeEvaluatorForLinkableInstances(Generic[SourceDataSetT]):
 
                 if identifier_instance is None:
                     raise RuntimeError(f"Could not find identifier instance with name ({identifier_spec_in_node})")
-                ident = self._data_source_semantics.get_data_source_element(identifier_instance.defined_from[0])
+
+                assert (
+                    len(identifier_instance.defined_from) == 1
+                ), f"Did not get exactly 1 defined_from in {identifier_instance}"
+
+                ident = self._data_source_semantics.get_identifier_in_data_source(identifier_instance.defined_from[0])
                 if ident is None:
                     raise RuntimeError(f"Invalid DataSourceElementReference {identifier_instance.defined_from[0]}")
                 if ident.type not in (IdentifierType.PRIMARY, IdentifierType.UNIQUE):
@@ -189,7 +194,8 @@ class NodeEvaluatorForLinkableInstances(Generic[SourceDataSetT]):
                     # but since we're doing all left joins now, it's been left out.
 
                     required_identifier_matches_data_set_identifier = (
-                        needed_linkable_spec.identifier_links[0] == linkless_identifier_spec_in_node
+                        LinklessIdentifierSpec.from_reference(needed_linkable_spec.identifier_links[0])
+                        == linkless_identifier_spec_in_node
                     )
                     needed_linkable_spec_in_node = (
                         needed_linkable_spec.without_first_identifier_link() in linkable_specs_in_node
@@ -262,7 +268,7 @@ class NodeEvaluatorForLinkableInstances(Generic[SourceDataSetT]):
     def evaluate_node(
         self,
         start_node: BaseOutput[SourceDataSetT],
-        required_linkable_specs: List[LinkableInstanceSpec],
+        required_linkable_specs: Sequence[LinkableInstanceSpec],
     ) -> LinkableInstanceSatisfiabilityEvaluation:
         """Evaluates if the "required_linkable_specs" can be realized by joining the "start_node" with other nodes.
 
@@ -291,7 +297,8 @@ class NodeEvaluatorForLinkableInstances(Generic[SourceDataSetT]):
             is_local = required_linkable_spec in data_set_linkable_specs
             is_unjoinable = (
                 len(required_linkable_spec.identifier_links) == 0
-                or required_linkable_spec.identifier_links[0] not in data_set_linkable_specs
+                or LinklessIdentifierSpec.from_reference(required_linkable_spec.identifier_links[0])
+                not in data_set_linkable_specs
             )
             if is_local:
                 local_linkable_specs.append(required_linkable_spec)
