@@ -10,9 +10,9 @@ from typing import List, TypeVar, Generic, Optional, Sequence, Tuple, Union
 import jinja2
 
 from metricflow.constraints.time_constraint import TimeRangeConstraint
-from metricflow.dag.mf_dag import DagNode, DisplayedProperty, MetricFlowDag, NodeId
 from metricflow.dag.id_generation import (
     DATAFLOW_NODE_AGGREGATE_MEASURES_ID_PREFIX,
+    DATAFLOW_NODE_SEMI_ADDITIVE_JOIN_ID_PREFIX,
     DATAFLOW_NODE_COMPUTE_METRICS_ID_PREFIX,
     DATAFLOW_NODE_JOIN_AGGREGATED_MEASURES_BY_GROUPBY_COLUMNS_PREFIX,
     DATAFLOW_NODE_JOIN_SELF_OVER_TIME_RANGE_ID_PREFIX,
@@ -24,25 +24,30 @@ from metricflow.dag.id_generation import (
     DATAFLOW_NODE_WRITE_TO_RESULT_DATAFRAME_ID_PREFIX,
     DATAFLOW_NODE_COMBINE_METRICS_ID_PREFIX,
     DATAFLOW_NODE_CONSTRAIN_TIME_RANGE_ID_PREFIX,
+    DATAFLOW_NODE_SET_MEASURE_AGGREGATION_TIME,
 )
+from metricflow.dag.mf_dag import DagNode, DisplayedProperty, MetricFlowDag, NodeId
 from metricflow.dataflow.builder.partitions import (
     PartitionDimensionJoinDescription,
     PartitionTimeDimensionJoinDescription,
 )
 from metricflow.dataflow.sql_table import SqlTable
 from metricflow.dataset.dataset import DataSet
+from metricflow.model.objects.metric import CumulativeMetricWindow
+from metricflow.aggregation_properties import AggregationType
 from metricflow.object_utils import pformat_big_objects
+from metricflow.references import TimeDimensionReference
 from metricflow.specs import (
+    MetricInputMeasureSpec,
     OrderBySpec,
     InstanceSpec,
     MetricSpec,
     LinklessIdentifierSpec,
-    TimeDimensionReference,
+    TimeDimensionSpec,
     SpecWhereClauseConstraint,
 )
-from metricflow.visitor import Visitable, VisitorOutputT
-from metricflow.model.objects.metric import CumulativeMetricWindow
 from metricflow.time.time_granularity import TimeGranularity
+from metricflow.visitor import Visitable, VisitorOutputT
 
 # The type of data set that is flowing out of the source nodes
 SourceDataSetT = TypeVar("SourceDataSetT", bound=DataSet)
@@ -144,6 +149,16 @@ class DataflowPlanNodeVisitor(Generic[SourceDataSetT, VisitorOutputT], ABC):
     def visit_join_over_time_range_node(self, node: JoinOverTimeRangeNode[SourceDataSetT]) -> VisitorOutputT:  # noqa: D
         pass
 
+    @abstractmethod
+    def visit_semi_additive_join_node(self, node: SemiAdditiveJoinNode[SourceDataSetT]) -> VisitorOutputT:  # noqa: D
+        pass
+
+    @abstractmethod
+    def visit_metric_time_dimension_transform_node(  # noqa: D
+        self, node: MetricTimeDimensionTransformNode[SourceDataSetT]
+    ) -> VisitorOutputT:
+        pass
+
 
 class BaseOutput(Generic[SourceDataSetT], DataflowPlanNode[SourceDataSetT], ABC):
     """A node that outputs data in a "base" format.
@@ -267,7 +282,7 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
     def __init__(
         self,
         parent_node: BaseOutput[SourceDataSetT],
-        primary_time_dimension_reference: TimeDimensionReference,
+        metric_time_dimension_reference: TimeDimensionReference,
         window: Optional[CumulativeMetricWindow],
         grain_to_date: Optional[TimeGranularity],
         node_id: Optional[NodeId] = None,
@@ -277,7 +292,8 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
 
         Args:
             parent_node: node with standard output
-            primary_time_dimension_reference: primary time dimension reference
+            metric_time_dimension_reference: the name of the virtual time dimension used in queries to refer to the time
+            dimension that each measure should be aggregated by.
             window: time window to join over
             grain_to_date: indicates time range should start from the beginning of this time granularity (eg month to day)
             node_id: Override the node ID with this value
@@ -286,7 +302,7 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
         self._parent_node = parent_node
         self._grain_to_date = grain_to_date
         self._window = window
-        self._primary_time_dimension_reference = primary_time_dimension_reference
+        self._metric_time_dimension_reference = metric_time_dimension_reference
         self.time_range_constraint = time_range_constraint
 
         # Doing a list comprehension throws a type error, so doing it this way.
@@ -301,8 +317,8 @@ class JoinOverTimeRangeNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT])
         return visitor.visit_join_over_time_range_node(self)
 
     @property
-    def primary_time_dimension_reference(self) -> TimeDimensionReference:  # noqa: D
-        return self._primary_time_dimension_reference
+    def metric_time_dimension_reference(self) -> TimeDimensionReference:  # noqa: D
+        return self._metric_time_dimension_reference
 
     @property
     def grain_to_date(self) -> Optional[TimeGranularity]:  # noqa: D
@@ -335,10 +351,23 @@ class AggregatedMeasuresOutput(Generic[SourceDataSetT], BaseOutput[SourceDataSet
 
 
 class AggregateMeasuresNode(Generic[SourceDataSetT], AggregatedMeasuresOutput[SourceDataSetT]):
-    """A node that aggregates the measures by the associated group by elements."""
+    """A node that aggregates the measures by the associated group by elements.
 
-    def __init__(self, parent_node: BaseOutput) -> None:  # noqa: D
+    In the event that one or more of the aggregated input measures has an alias assigned to it, any output query
+    resulting from an operation on this node must apply the alias and transform the measure instances accordingly,
+    otherwise this join could produce a query with two identically named measure columns with, e.g., different
+    constraints applied to the measure.
+    """
+
+    def __init__(self, parent_node: BaseOutput, metric_input_measure_specs: Tuple[MetricInputMeasureSpec, ...]) -> None:
+        """Initializer for AggregateMeasuresNode
+
+        The input measure specs are required for downstream nodes to be aware of any input measures with
+        user-provided aliases, such as we might encounter with constrained and unconstrained versions of the
+        same input measure.
+        """
         self._parent_node = parent_node
+        self._metric_input_measure_specs = metric_input_measure_specs
 
         super().__init__(node_id=self.create_unique_id(), parent_nodes=[self._parent_node])
 
@@ -357,6 +386,14 @@ class AggregateMeasuresNode(Generic[SourceDataSetT], AggregatedMeasuresOutput[So
     def parent_node(self) -> BaseOutput:  # noqa: D
         return self._parent_node
 
+    @property
+    def metric_input_measure_specs(self) -> Tuple[MetricInputMeasureSpec, ...]:
+        """Iterable of specs for measure inputs to downstream metrics.
+
+        Used for assigning aliases to output columns produced by aggregated measures.
+        """
+        return self._metric_input_measure_specs
+
 
 class JoinAggregatedMeasuresByGroupByColumnsNode(Generic[SourceDataSetT], AggregatedMeasuresOutput[SourceDataSetT]):
     """A node that joins aggregated measures with group by elements.
@@ -402,6 +439,118 @@ class JoinAggregatedMeasuresByGroupByColumnsNode(Generic[SourceDataSetT], Aggreg
         ]
 
 
+class SemiAdditiveJoinNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
+    """A node that performs a row filter by aggregating a given non-additive dimension.
+
+    This is designed to filter a dataset down to singular non-additive time dimension values by aggregating
+    the time dimension with MAX/MIN then joining back to the original dataset on that aggregated value.
+    Additionally, an optional sequence of identifiers can be passed in to group by and join on during the filtering.
+
+    For example, if we have a data set that includes "account_balances,user,date", we can build a
+    "latest_account_balance_by_user" data set using this node by filtering the data set such that for each user,
+    we get the MAX(date) and return the account_balance corresponding to that date.
+
+    Data transformation example,
+    | date       | account_balance | user |                             | date       | account_balance | user |
+    |:-----------|-----------------|-----:|     identifier_specs:       |:-----------|-----------------|-----:|
+    | 2019-12-31 |            1000 |    u1|       - user                | 2020-01-03 |            2000 |    u1|
+    | 2020-01-03 |            2000 |    u1| ->  time_dimension_spec: -> | 2020-01-12              1500 |    u2|
+    | 2020-01-09 |            3000 |    u2|       - date                | 2020-01-12 |            1000 |    u3|
+    | 2020-01-12 |            1500 |    u2|     agg_by_function:
+    | 2020-01-12 |            1000 |    u3|       - MAX
+
+
+    Similarly, if we don't provide any identifier_specs, it would end up performing the aggregation filter only on the
+    time_dimension_spec without any grouping by any identifiers.
+
+    Data transformation example,
+    | date       | account_balance | user |                             | date       | account_balance |
+    |:-----------|-----------------|-----:|     identifier_specs:       |:-----------|----------------:|
+    | 2019-12-31 |            1000 |    u1|                             | 2020-01-12 |            2500 |
+    | 2020-01-03 |            2000 |    u1| ->  time_dimension_spec: ->
+    | 2020-01-09 |            3000 |    u2|       - date
+    | 2020-01-12 |            1500 |    u2|     agg_by_function:
+    | 2020-01-12 |            1000 |    u3|       - MAX
+
+
+    Additionally, we can aggregate against 'windows' of a dataset. For example, if we group by a non_additive time dimension,
+    we perform the MIN/MAX filtering on the granularity window provided in that group by instead of the full time range.
+
+    Data transformation example,
+    | date       | account_balance | user |                                  | date       | account_balance |
+    |:-----------|-----------------|-----:|     identifier_specs:            |:-----------|----------------:|
+    | 2019-12-31 |            1500 |    u1|     time_dimension_spec:         | 2019-12-31 |            1500 |
+    | 2020-01-03 |            2000 |    u1| ->    - date                  -> | 2020-01-07 |            3000 |
+    | 2020-01-09 |            3000 |    u2|     agg_by_function:             | 2020-01-14 |            3250 |
+    | 2020-01-12 |            1500 |    u2|       - MIN
+    | 2020-01-14 |            1250 |    u3|     queried_time_dimension_spec:
+    | 2020-01-14 |            2000 |    u2|       - date__week
+    | 2020-01-15 |            4000 |    u1|
+    """
+
+    def __init__(
+        self,
+        parent_node: BaseOutput[SourceDataSetT],
+        identifier_specs: Sequence[LinklessIdentifierSpec],
+        time_dimension_spec: TimeDimensionSpec,
+        agg_by_function: AggregationType,
+        queried_time_dimension_spec: Optional[TimeDimensionSpec] = None,
+    ) -> None:
+        """Constructor.
+
+        Args:
+            parent_node: node with standard output
+            identifier_specs: the identifiers to group the join by
+            time_dimension_spec: the time dimension used for row filtering via an aggregation
+            agg_by_function: the aggregation function used on the time dimension
+            queried_time_dimension_spec: The group by provided in the query used to build the windows we want to filter on.
+        """
+        self._parent_node = parent_node
+        self._identifier_specs = identifier_specs
+        self._time_dimension_spec = time_dimension_spec
+        self._agg_by_function = agg_by_function
+        self._queried_time_dimension_spec = queried_time_dimension_spec
+
+        # Doing a list comprehension throws a type error, so doing it this way.
+        parent_nodes: List[DataflowPlanNode[SourceDataSetT]] = [self._parent_node]
+        super().__init__(node_id=self.create_unique_id(), parent_nodes=parent_nodes)
+
+    @classmethod
+    def id_prefix(cls) -> str:  # noqa: D
+        return DATAFLOW_NODE_SEMI_ADDITIVE_JOIN_ID_PREFIX
+
+    def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:  # noqa: D
+        return visitor.visit_semi_additive_join_node(self)
+
+    @property
+    def description(self) -> str:  # noqa: D
+        return f"""Join on {self.agg_by_function.name}({self.time_dimension_spec.element_name}) and {[i.element_name for i in self.identifier_specs]} grouping by {self.queried_time_dimension_spec.element_name if self.queried_time_dimension_spec else None}"""
+
+    @property
+    def parent_node(self) -> BaseOutput[SourceDataSetT]:  # noqa: D
+        return self._parent_node
+
+    @property
+    def identifier_specs(self) -> Sequence[LinklessIdentifierSpec]:  # noqa: D
+        return self._identifier_specs
+
+    @property
+    def time_dimension_spec(self) -> TimeDimensionSpec:  # noqa: D
+        return self._time_dimension_spec
+
+    @property
+    def agg_by_function(self) -> AggregationType:  # noqa: D
+        return self._agg_by_function
+
+    @property
+    def queried_time_dimension_spec(self) -> Optional[TimeDimensionSpec]:  # noqa: D
+        return self._queried_time_dimension_spec
+
+    @property
+    def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
+        return super().displayed_properties
+
+
 class ComputedMetricsOutput(Generic[SourceDataSetT], BaseOutput[SourceDataSetT], ABC):
     """A node that outputs data that contains metrics computed from measures."""
 
@@ -507,6 +656,53 @@ class OrderByLimitNode(Generic[SourceDataSetT], ComputedMetricsOutput[SourceData
         return self._parent_node
 
 
+class MetricTimeDimensionTransformNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
+    """A node transforms the input data set so that it contains the metric time dimension and relevant measures.
+
+    The metric time dimension is used later to aggregate all measures in the data set.
+
+    Input: a data set containing measures along with the associated aggregation time dimension.
+
+    Output: a data set similar to the input data set, but includes the configured aggregation time dimension as the
+    metric time dimension and only contains measures that are defined to use it.
+    """
+
+    def __init__(  # noqa: D
+        self,
+        parent_node: BaseOutput[SourceDataSetT],
+        aggregation_time_dimension_reference: TimeDimensionReference,
+    ) -> None:
+        self._aggregation_time_dimension_reference = aggregation_time_dimension_reference
+        self._parent_node = parent_node
+        super().__init__(node_id=self.create_unique_id(), parent_nodes=[parent_node])
+
+    @classmethod
+    def id_prefix(cls) -> str:  # noqa: D
+        return DATAFLOW_NODE_SET_MEASURE_AGGREGATION_TIME
+
+    @property
+    def aggregation_time_dimension_reference(self) -> TimeDimensionReference:
+        """The time dimension that measures in the input should be aggregated to."""
+        return self._aggregation_time_dimension_reference
+
+    def accept(self, visitor: DataflowPlanNodeVisitor[SourceDataSetT, VisitorOutputT]) -> VisitorOutputT:  # noqa: D
+        return visitor.visit_metric_time_dimension_transform_node(self)
+
+    @property
+    def description(self) -> str:  # noqa: D
+        return f"Metric Time Dimension '{self.aggregation_time_dimension_reference.element_name}'" ""
+
+    @property
+    def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
+        return super().displayed_properties + [
+            DisplayedProperty("aggregation_time_dimension", self.aggregation_time_dimension_reference.element_name)
+        ]
+
+    @property
+    def parent_node(self) -> BaseOutput[SourceDataSetT]:  # noqa: D
+        return self._parent_node
+
+
 class SinkNodeVisitor(Generic[SourceDataSetT, VisitorOutputT], ABC):
     """Similar to DataflowPlanNodeVisitor, but only for sink nodes."""
 
@@ -618,8 +814,10 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
         self,
         parent_node: BaseOutput[SourceDataSetT],
         include_specs: Sequence[InstanceSpec],
+        replace_description: Optional[str] = None,
     ) -> None:
         self._include_specs = include_specs
+        self._replace_description = replace_description
         super().__init__(node_id=self.create_unique_id(), parent_nodes=[parent_node])
 
     @classmethod
@@ -636,6 +834,9 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
 
     @property
     def description(self) -> str:  # noqa: D
+        if self._replace_description:
+            return self._replace_description
+
         formatted_str = textwrap.indent(
             pformat_big_objects([x.qualified_name for x in self._include_specs]), prefix="  "
         )
@@ -643,9 +844,12 @@ class FilterElementsNode(Generic[SourceDataSetT], BaseOutput[SourceDataSetT]):
 
     @property
     def displayed_properties(self) -> List[DisplayedProperty]:  # noqa: D
-        return super().displayed_properties + [
-            DisplayedProperty("include_spec", include_spec) for include_spec in self._include_specs
-        ]
+        additional_properties = []
+        if not self._replace_description:
+            additional_properties = [
+                DisplayedProperty("include_spec", include_spec) for include_spec in self._include_specs
+            ]
+        return super().displayed_properties + additional_properties
 
     @property
     def parent_node(self) -> DataflowPlanNode:  # noqa: D
@@ -767,3 +971,8 @@ class DataflowPlan(Generic[SourceDataSetT], MetricFlowDag[SinkOutput[SourceDataS
     @property
     def sink_output_nodes(self) -> List[SinkOutput[SourceDataSetT]]:  # noqa: D
         return self._sink_output_nodes
+
+    @property
+    def sink_output_node(self) -> SinkOutput[SourceDataSetT]:  # noqa: D
+        assert len(self._sink_output_nodes) == 1
+        return self._sink_output_nodes[0]
