@@ -3,8 +3,10 @@ import re
 from dataclasses import dataclass
 from typing import Optional, List, Tuple, Sequence
 
+from metricflow.aggregation_properties import AggregationState
 from metricflow.dag.id_generation import IdGeneratorRegistry
 from metricflow.dataflow.sql_table import SqlTable
+from metricflow.dataset.data_source_adapter import DataSourceDataSet
 from metricflow.instances import (
     MeasureInstance,
     DataSourceElementReference,
@@ -13,19 +15,19 @@ from metricflow.instances import (
     IdentifierInstance,
     InstanceSet,
     DataSourceReference,
-    AggregationState,
 )
 from metricflow.model.objects.data_source import DataSource
 from metricflow.model.objects.elements.dimension import Dimension, DimensionType
 from metricflow.model.objects.elements.identifier import Identifier, IdentifierType
 from metricflow.model.objects.elements.measure import Measure
+from metricflow.model.spec_converters import MeasureConverter
 from metricflow.specs import (
     TimeDimensionSpec,
-    MeasureSpec,
     DimensionSpec,
     IdentifierSpec,
     ColumnAssociationResolver,
-    LinklessIdentifierSpec,
+    DEFAULT_TIME_GRANULARITY,
+    IdentifierReference,
 )
 from metricflow.sql.sql_exprs import (
     SqlStringExpression,
@@ -42,7 +44,6 @@ from metricflow.sql.sql_plan import (
     SqlSelectQueryFromClauseNode,
 )
 from metricflow.time.time_granularity import TimeGranularity
-from metricflow.dataset.data_source_adapter import DataSourceDataSet
 
 logger = logging.getLogger(__name__)
 
@@ -75,11 +76,11 @@ class DataSourceToDataSetConverter:
         self,
         data_source_name: str,
         dimension: Dimension,
-        identifier_links: Tuple[LinklessIdentifierSpec, ...],
+        identifier_links: Tuple[IdentifierReference, ...],
     ) -> DimensionInstance:
         """Create a dimension instance from the dimension object in the model."""
         dimension_spec = DimensionSpec(
-            element_name=dimension.name.element_name,
+            element_name=dimension.reference.element_name,
             identifier_links=identifier_links,
         )
         column_associations = dimension_spec.column_associations(self._column_association_resolver)
@@ -90,7 +91,7 @@ class DataSourceToDataSetConverter:
             defined_from=(
                 DataSourceElementReference(
                     data_source_name=data_source_name,
-                    element_name=dimension.name.element_name,
+                    element_name=dimension.reference.element_name,
                 ),
             ),
         )
@@ -99,12 +100,12 @@ class DataSourceToDataSetConverter:
         self,
         data_source_name: str,
         time_dimension: Dimension,
-        identifier_links: Tuple[LinklessIdentifierSpec, ...],
-        time_granularity: Optional[TimeGranularity] = None,
+        identifier_links: Tuple[IdentifierReference, ...],
+        time_granularity: TimeGranularity = DEFAULT_TIME_GRANULARITY,
     ) -> TimeDimensionInstance:
         """Create a time dimension instance from the dimension object from a data source in the model."""
         time_dimension_spec = TimeDimensionSpec(
-            element_name=time_dimension.name.element_name,
+            element_name=time_dimension.reference.element_name,
             identifier_links=identifier_links,
             time_granularity=time_granularity,
         )
@@ -117,7 +118,7 @@ class DataSourceToDataSetConverter:
             defined_from=(
                 DataSourceElementReference(
                     data_source_name=data_source_name,
-                    element_name=time_dimension.name.element_name,
+                    element_name=time_dimension.reference.element_name,
                 ),
             ),
         )
@@ -126,11 +127,11 @@ class DataSourceToDataSetConverter:
         self,
         data_source_name: str,
         identifier: Identifier,
-        identifier_links: Tuple[LinklessIdentifierSpec, ...],
+        identifier_links: Tuple[IdentifierReference, ...],
     ) -> IdentifierInstance:
         """Create an identifier instance from the identifier object from a data sourcein the model."""
         identifier_spec = IdentifierSpec(
-            element_name=identifier.name.element_name,
+            element_name=identifier.reference.element_name,
             identifier_links=identifier_links,
         )
         column_associations = identifier_spec.column_associations(self._column_association_resolver)
@@ -141,7 +142,7 @@ class DataSourceToDataSetConverter:
             defined_from=(
                 DataSourceElementReference(
                     data_source_name=data_source_name,
-                    element_name=identifier.name.element_name,
+                    element_name=identifier.reference.element_name,
                 ),
             ),
         )
@@ -183,17 +184,14 @@ class DataSourceToDataSetConverter:
         measure_instances = []
         select_columns = []
         for measure in measures or []:
-            measure_spec = MeasureSpec(
-                element_name=measure.name.element_name,
-            )
+            measure_spec = MeasureConverter.convert_to_measure_spec(measure=measure)
             measure_instance = MeasureInstance(
                 associated_columns=measure_spec.column_associations(self._column_association_resolver),
-                source_time_dimension_reference=measure_time_dimension_spec.reference,
                 spec=measure_spec,
                 defined_from=(
                     DataSourceElementReference(
                         data_source_name=data_source_name,
-                        element_name=measure.name.element_name,
+                        element_name=measure.reference.element_name,
                     ),
                 ),
                 aggregation_state=AggregationState.NON_AGGREGATED,
@@ -203,7 +201,7 @@ class DataSourceToDataSetConverter:
                 SqlSelectColumn(
                     expr=DataSourceToDataSetConverter._make_element_sql_expr(
                         table_alias=table_alias,
-                        element_name=measure.name.element_name,
+                        element_name=measure.reference.element_name,
                         element_expr=measure.expr,
                     ),
                     column_alias=measure_instance.associated_column.column_name,
@@ -216,7 +214,7 @@ class DataSourceToDataSetConverter:
         self,
         data_source_name: str,
         dimensions: Sequence[Dimension],
-        identifier_links: Tuple[LinklessIdentifierSpec, ...],
+        identifier_links: Tuple[IdentifierReference, ...],
         table_alias: str,
     ) -> DimensionConversionResult:
         dimension_instances = []
@@ -235,7 +233,7 @@ class DataSourceToDataSetConverter:
                     SqlSelectColumn(
                         expr=DataSourceToDataSetConverter._make_element_sql_expr(
                             table_alias=table_alias,
-                            element_name=dimension.name.element_name,
+                            element_name=dimension.reference.element_name,
                             element_expr=dimension.expr,
                         ),
                         column_alias=dimension_instance.associated_column.column_name,
@@ -258,7 +256,7 @@ class DataSourceToDataSetConverter:
                     SqlSelectColumn(
                         expr=DataSourceToDataSetConverter._make_element_sql_expr(
                             table_alias=table_alias,
-                            element_name=dimension.name.element_name,
+                            element_name=dimension.reference.element_name,
                             element_expr=dimension.expr,
                         ),
                         column_alias=time_dimension_instance.associated_column.column_name,
@@ -282,7 +280,7 @@ class DataSourceToDataSetConverter:
                                     time_granularity=time_granularity,
                                     arg=DataSourceToDataSetConverter._make_element_sql_expr(
                                         table_alias=table_alias,
-                                        element_name=dimension.name.element_name,
+                                        element_name=dimension.reference.element_name,
                                         element_expr=dimension.expr,
                                     ),
                                 ),
@@ -302,7 +300,7 @@ class DataSourceToDataSetConverter:
         self,
         data_source_name: str,
         identifiers: Sequence[Identifier],
-        identifier_links: Tuple[LinklessIdentifierSpec, ...],
+        identifier_links: Tuple[IdentifierReference, ...],
         table_alias: str,
     ) -> Tuple[Sequence[IdentifierInstance], Sequence[SqlSelectColumn]]:
         identifier_instances = []
@@ -310,10 +308,7 @@ class DataSourceToDataSetConverter:
         for identifier in identifiers or []:
             # We don't want to create something like user_id__user_id, so skip if the link is the same as the
             # identifier.
-            if (
-                len(identifier_links) == 1
-                and LinklessIdentifierSpec.from_element_name(identifier.name.element_name) == identifier_links[0]
-            ):
+            if len(identifier_links) == 1 and identifier.reference == identifier_links[0]:
                 continue
 
             identifier_instance = self._create_identifier_instance(
@@ -331,7 +326,7 @@ class DataSourceToDataSetConverter:
                     expr = sub_id.expr
                     if expr is None:
                         assert sub_id.name is not None
-                        expr = sub_id.name.element_name
+                        expr = sub_id.name
                     sub_id_name = sub_id.ref or sub_id.name
                     assert sub_id_name, f"Sub-identifier {sub_id} must have 'name' or 'ref' defined"
 
@@ -350,7 +345,7 @@ class DataSourceToDataSetConverter:
                     SqlSelectColumn(
                         expr=DataSourceToDataSetConverter._make_element_sql_expr(
                             table_alias=table_alias,
-                            element_name=identifier.name.element_name,
+                            element_name=identifier.reference.element_name,
                             element_expr=identifier.expr,
                         ),
                         column_alias=identifier_instance.associated_column.column_name,
@@ -373,7 +368,7 @@ class DataSourceToDataSetConverter:
         ), f"Primary time dimension missing time granularity: {primary_time_dimension}"
 
         return TimeDimensionSpec(
-            element_name=primary_time_dimension.name.element_name,
+            element_name=primary_time_dimension.reference.element_name,
             identifier_links=(),
             time_granularity=primary_time_dimension.type_params.time_granularity,
         )
@@ -406,12 +401,10 @@ class DataSourceToDataSetConverter:
         # e.g. in the "users" data source, with the "country" dimension and the "user_id" identifier,
         # the dimensions "country" and "user_id__country" both mean the same thing. To make matching easier, create both
         # instances in the instance set. We'll create a different instance for each "possible_identifier_links".
-        possible_identifier_links: List[Tuple[LinklessIdentifierSpec, ...]] = [()]
+        possible_identifier_links: List[Tuple[IdentifierReference, ...]] = [()]
         for identifier in data_source.identifiers:
             if identifier.type in (IdentifierType.PRIMARY, IdentifierType.UNIQUE):
-                possible_identifier_links.append(
-                    (LinklessIdentifierSpec.from_element_name(element_name=identifier.name.element_name),)
-                )
+                possible_identifier_links.append((identifier.reference,))
 
         # Handle dimensions
         conversion_results = [
