@@ -9,9 +9,11 @@ from typing import Optional, List, Sequence
 
 import pandas as pd
 
-from metricflow.configuration.constants import CONFIG_DWH_SCHEMA
+from metricflow.configuration.constants import CONFIG_DBT_PROFILE, CONFIG_DBT_REPO, CONFIG_DBT_TARGET, CONFIG_DWH_SCHEMA
 from metricflow.configuration.yaml_handler import YamlFileHandler
 from metricflow.dataflow.builder.dataflow_plan_builder import DataflowPlanBuilder
+from metricflow.dataflow.builder.node_data_set import DataflowPlanNodeOutputDataSetResolver
+from metricflow.dataflow.builder.source_node import SourceNodeBuilder
 from metricflow.dataflow.dataflow_plan import DataflowPlan
 from metricflow.dataflow.sql_table import SqlTable
 from metricflow.dataset.convert_data_source import DataSourceToDataSetConverter
@@ -22,15 +24,16 @@ from metricflow.execution.execution_plan import ExecutionPlan, SqlQuery
 from metricflow.execution.execution_plan_to_text import execution_plan_to_text
 from metricflow.execution.executor import SequentialPlanExecutor
 from metricflow.model.semantic_model import SemanticModel
+from metricflow.model.semantics.linkable_spec_resolver import LinkableElementProperties
 from metricflow.object_utils import pformat_big_objects, random_id
 from metricflow.plan_conversion.column_resolver import DefaultColumnAssociationResolver
 from metricflow.plan_conversion.dataflow_to_execution import DataflowToExecutionPlanConverter
 from metricflow.plan_conversion.dataflow_to_sql import DataflowToSqlQueryPlanConverter
-from metricflow.plan_conversion.time_spine import TimeSpineSource
+from metricflow.plan_conversion.time_spine import TimeSpineSource, TimeSpineTableBuilder
 from metricflow.protocols.sql_client import SqlClient
 from metricflow.query.query_parser import MetricFlowQueryParser
-from metricflow.specs import ColumnAssociationResolver
-from metricflow.specs import MetricSpec, MetricFlowQuerySpec
+from metricflow.specs import ColumnAssociationResolver, MetricFlowQuerySpec
+from metricflow.references import MetricReference
 from metricflow.sql.optimizer.optimization_levels import SqlQueryOptimizationLevel
 from metricflow.time.time_source import TimeSource
 from metricflow.dataset.data_source_adapter import DataSourceDataSet
@@ -141,6 +144,17 @@ class MetricFlowExplainResult:
 
         return sql_query
 
+    @property
+    def rendered_sql_without_descriptions(self) -> SqlQuery:
+        """Return the SQL query without the inline descriptions."""
+        sql_query = self.rendered_sql
+        return SqlQuery(
+            sql_query="\n".join(
+                filter(lambda line: not line.strip().startswith("--"), sql_query.sql_query.split("\n"))
+            ),
+            bind_parameters=sql_query.bind_parameters,
+        )
+
 
 class AbstractMetricFlowEngine(ABC):
     """Query interface for clients"""
@@ -258,7 +272,25 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
     def from_config(handler: YamlFileHandler) -> MetricFlowEngine:
         """Initialize MetricFlowEngine via yaml config file."""
         sql_client = make_sql_client_from_config(handler)
-        semantic_model = SemanticModel(build_user_configured_model_from_config(handler))
+
+        # Ideally we should put this getting of of CONFIG_DBT_REPO in a helper
+        dbt_repo = handler.get_value(CONFIG_DBT_REPO) or ""
+        if dbt_repo.lower() in ["yes", "y", "true", "t", "1"]:
+            # This import results in eventually importing dbt, and dbt is an
+            # optional dep meaning it isn't guaranteed to be installed. If the
+            # import is at the top ofthe file MetricFlow will blow up if dbt
+            # isn't installed. Thus by importing it here, we only run into the
+            # exception if this conditional is hit without dbt installed
+            from metricflow.engine.utils import build_user_configured_model_from_dbt_config
+
+            dbt_profile = handler.get_value(CONFIG_DBT_PROFILE)
+            dbt_target = handler.get_value(CONFIG_DBT_TARGET)
+
+            semantic_model = SemanticModel(
+                build_user_configured_model_from_dbt_config(handler=handler, profile=dbt_profile, target=dbt_target)
+            )
+        else:
+            semantic_model = SemanticModel(build_user_configured_model_from_config(handler))
         system_schema = not_empty(handler.get_value(CONFIG_DWH_SCHEMA), CONFIG_DWH_SCHEMA, handler.url)
         return MetricFlowEngine(
             semantic_model=semantic_model,
@@ -290,7 +322,10 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
             DefaultColumnAssociationResolver(semantic_model)
         )
         self._time_source = time_source
-        self._time_spine_source = time_spine_source or TimeSpineSource(sql_client, schema_name=system_schema)
+        self._time_spine_source = time_spine_source or TimeSpineSource(schema_name=system_schema)
+        self._time_spine_table_builder = TimeSpineTableBuilder(
+            time_spine_source=self._time_spine_source, sql_client=self._sql_client
+        )
 
         self._schema = system_schema
 
@@ -301,14 +336,18 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
             self._source_data_sets.append(data_set)
             logger.info(f"Created source dataset from data source '{data_source.name}'")
 
-        self._primary_time_dimension_reference = (
-            self._semantic_model.data_source_semantics.primary_time_dimension_reference
+        source_node_builder = SourceNodeBuilder(self._semantic_model)
+        source_nodes = source_node_builder.create_from_data_sets(self._source_data_sets)
+
+        node_output_resolver = DataflowPlanNodeOutputDataSetResolver[DataSourceDataSet](
+            column_association_resolver=DefaultColumnAssociationResolver(semantic_model),
+            semantic_model=semantic_model,
+            time_spine_source=self._time_spine_source,
         )
 
-        self._dataflow_plan_builder = DataflowPlanBuilder(
-            data_sets=self._source_data_sets,
+        self._dataflow_plan_builder = DataflowPlanBuilder[DataSourceDataSet](
+            source_nodes=source_nodes,
             semantic_model=self._semantic_model,
-            primary_time_dimension_reference=self._primary_time_dimension_reference,
             time_spine_source=self._time_spine_source,
         )
         self._to_sql_query_plan_converter = DataflowToSqlQueryPlanConverter[DataSourceDataSet](
@@ -316,7 +355,7 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
             semantic_model=self._semantic_model,
             time_spine_source=self._time_spine_source,
         )
-        self._to_execution_plan_converter = DataflowToExecutionPlanConverter(
+        self._to_execution_plan_converter = DataflowToExecutionPlanConverter[DataSourceDataSet](
             sql_plan_converter=self._to_sql_query_plan_converter,
             sql_plan_renderer=self._sql_client.sql_engine_attributes.sql_query_plan_renderer,
             sql_client=sql_client,
@@ -325,7 +364,8 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
 
         self._query_parser = MetricFlowQueryParser(
             model=self._semantic_model,
-            primary_time_dimension_reference=self._primary_time_dimension_reference,
+            source_nodes=source_nodes,
+            node_output_resolver=node_output_resolver,
         )
 
     def _get_materialization_by_name(self, materialization_name: str) -> Optional[Materialization]:
@@ -384,8 +424,10 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
         )
         logger.info(f"Query spec is:\n{pformat_big_objects(query_spec)}")
 
-        if self._semantic_model.metric_semantics.contains_cumulative_metric(query_spec.metric_specs):
-            self._time_spine_source.create_if_necessary()
+        if self._semantic_model.metric_semantics.contains_cumulative_metric(
+            tuple(m.as_reference for m in query_spec.metric_specs)
+        ):
+            self._time_spine_table_builder.create_if_necessary()
             time_constraint_updated = False
             if not mf_query_request.time_constraint_start:
                 time_constraint_start = self._time_source.get_time() - datetime.timedelta(days=365)
@@ -442,10 +484,14 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
         return [
             Dimension(name=dim.qualified_name)
             for dim in self._semantic_model.metric_semantics.element_specs_for_metrics(
-                metric_specs=[MetricSpec(element_name=mname) for mname in metric_names],
-                dimensions_only=True,
-                exclude_derived_time_granularities=True,
-                exclude_local_linked_primary_time=True,
+                metric_references=[MetricReference(element_name=mname) for mname in metric_names],
+                without_any_property=frozenset(
+                    {
+                        LinkableElementProperties.IDENTIFIER,
+                        LinkableElementProperties.DERIVED_TIME_GRANULARITY,
+                        LinkableElementProperties.LOCAL_LINKED,
+                    }
+                ),
             )
         ]
 
@@ -460,8 +506,8 @@ class MetricFlowEngine(AbstractMetricFlowEngine):
 
     @log_call(module_name=__name__, telemetry_reporter=_telemetry_reporter)
     def list_metrics(self) -> List[Metric]:  # noqa: D
-        metric_specs = self._semantic_model.metric_semantics.metric_names
-        metrics = self._semantic_model.metric_semantics.get_metrics(metric_specs)
+        metric_references = self._semantic_model.metric_semantics.metric_references
+        metrics = self._semantic_model.metric_semantics.get_metrics(metric_references)
         return [
             Metric(
                 name=metric.name,
