@@ -2,9 +2,9 @@ from __future__ import annotations
 
 from typing import Optional
 
-from metricflow.model.objects.common import Element
-from metricflow.model.objects.utils import ParseableObject, HashableBaseModel
-from metricflow.specs import DimensionReference
+from metricflow.model.objects.base import HashableBaseModel, ModelWithMetadataParsing
+from metricflow.model.objects.common import Metadata
+from metricflow.references import DimensionReference, TimeDimensionReference
 from metricflow.time.time_granularity import TimeGranularity
 from metricflow.object_utils import ExtendedEnum
 
@@ -22,7 +22,7 @@ class DimensionType(ExtendedEnum):
         return self in [DimensionType.TIME]
 
 
-class DimensionTypeParams(HashableBaseModel, ParseableObject):
+class DimensionTypeParams(HashableBaseModel):
     """Dimension type params add additional context to some types (time) of dimensions"""
 
     is_primary: bool = False
@@ -31,14 +31,16 @@ class DimensionTypeParams(HashableBaseModel, ParseableObject):
     time_granularity: TimeGranularity
 
 
-class Dimension(HashableBaseModel, Element, ParseableObject):
+class Dimension(HashableBaseModel, ModelWithMetadataParsing):
     """Describes a dimension"""
 
-    name: DimensionReference
+    name: str
+    description: Optional[str]
     type: DimensionType
     is_partition: bool = False
     type_params: Optional[DimensionTypeParams]
     expr: Optional[str] = None
+    metadata: Optional[Metadata]
 
     @property
     def is_primary_time(self) -> bool:  # noqa: D
@@ -46,3 +48,12 @@ class Dimension(HashableBaseModel, Element, ParseableObject):
             return self.type_params.is_primary
 
         return False
+
+    @property
+    def reference(self) -> DimensionReference:  # noqa: D
+        return DimensionReference(element_name=self.name)
+
+    @property
+    def time_dimension_reference(self) -> TimeDimensionReference:  # noqa: D
+        assert self.type == DimensionType.TIME, f"Got type as {self.type} instead of {DimensionType.TIME}"
+        return TimeDimensionReference(element_name=self.name)
