@@ -31,13 +31,13 @@ from metricflow.references import (
     IdentifierReference,
     LinkableElementReference,
     MeasureReference,
+    MetricReference,
     TimeDimensionReference,
 )
 from metricflow.specs import (
     LinkableInstanceSpec,
     MeasureSpec,
     MetricInputMeasureSpec,
-    MetricSpec,
     NonAdditiveDimensionSpec,
 )
 
@@ -51,11 +51,11 @@ class MetricSemantics:  # noqa: D
         self, user_configured_model: UserConfiguredModel, data_source_semantics: DataSourceSemantics
     ) -> None:
         self._user_configured_model = user_configured_model
-        self._metrics: Dict[MetricSpec, Metric] = {}
+        self._metrics: Dict[MetricReference, Metric] = {}
         self._data_source_semantics = data_source_semantics
 
         # Dict from the name of the metric to the hash.
-        self._metric_hashes: Dict[MetricSpec, str] = {}
+        self._metric_hashes: Dict[MetricReference, str] = {}
 
         for metric in self._user_configured_model.metrics:
             self.add_metric(metric)
@@ -67,59 +67,61 @@ class MetricSemantics:  # noqa: D
 
     def element_specs_for_metrics(
         self,
-        metric_specs: List[MetricSpec],
+        metric_references: List[MetricReference],
         with_any_property: FrozenSet[LinkableElementProperties] = LinkableElementProperties.all_properties(),
         without_any_property: FrozenSet[LinkableElementProperties] = frozenset(),
     ) -> List[LinkableInstanceSpec]:
         """Dimensions common to all metrics requested (intersection)"""
 
         all_linkable_specs = self._linkable_spec_resolver.get_linkable_elements_for_metrics(
-            metric_specs=metric_specs,
+            metric_references=metric_references,
             with_any_of=with_any_property,
             without_any_of=without_any_property,
         ).as_spec_set
 
         return sorted(all_linkable_specs.as_tuple, key=lambda x: x.qualified_name)
 
-    def get_metrics(self, metric_names: List[MetricSpec]) -> List[Metric]:  # noqa: D
+    def get_metrics(self, metric_references: List[MetricReference]) -> List[Metric]:  # noqa: D
         res = []
-        for metric_name in metric_names:
-            if metric_name not in self._metrics:
-                raise MetricNotFoundError(f"Unable to find metric `{metric_name}`. Perhaps it has not been registered")
-            res.append(self._metrics[metric_name])
+        for metric_reference in metric_references:
+            if metric_reference not in self._metrics:
+                raise MetricNotFoundError(
+                    f"Unable to find metric `{metric_reference}`. Perhaps it has not been registered"
+                )
+            res.append(self._metrics[metric_reference])
 
         return res
 
     @property
-    def metric_names(self) -> List[MetricSpec]:  # noqa: D
+    def metric_references(self) -> List[MetricReference]:  # noqa: D
         return list(self._metrics.keys())
 
-    def get_metric(self, metric_name: MetricSpec) -> Metric:  # noqa:D
-        if metric_name not in self._metrics:
-            raise MetricNotFoundError(f"Unable to find metric `{metric_name}`. Perhaps it has not been registered")
-        return self._metrics[metric_name]
+    def get_metric(self, metric_reference: MetricReference) -> Metric:  # noqa:D
+        if metric_reference not in self._metrics:
+            raise MetricNotFoundError(f"Unable to find metric `{metric_reference}`. Perhaps it has not been registered")
+        return self._metrics[metric_reference]
 
     def add_metric(self, metric: Metric) -> None:
         """Add metric, validating presence of required measures"""
-        metric_spec = MetricSpec(element_name=metric.name)
-        if metric_spec in self._metrics:
+        metric_reference = MetricReference(element_name=metric.name)
+        if metric_reference in self._metrics:
             raise DuplicateMetricError(f"Metric `{metric.name}` has already been registered")
         for measure_reference in metric.measure_references:
             if measure_reference not in self._data_source_semantics.measure_references:
                 raise NonExistentMeasureError(
                     f"Metric `{metric.name}` references measure `{measure_reference}` which has not been registered"
                 )
-        self._metrics[metric_spec] = metric
-        self._metric_hashes[metric_spec] = metric.definition_hash
+        self._metrics[metric_reference] = metric
+        self._metric_hashes[metric_reference] = metric.definition_hash
 
     @property
     def valid_hashes(self) -> Set[str]:
         """Return all of the hashes of the metric definitions."""
         return set(self._metric_hashes.values())
 
-    def measures_for_metric(self, metric_spec: MetricSpec) -> Tuple[MetricInputMeasureSpec, ...]:
+    def measures_for_metric(self, metric_reference: MetricReference) -> Tuple[MetricInputMeasureSpec, ...]:
         """Return the measure specs required to compute the metric."""
-        metric = self.get_metric(metric_spec)
+        metric = self.get_metric(metric_reference)
         input_measure_specs: List[MetricInputMeasureSpec] = []
 
         for input_measure in metric.input_measures:
@@ -146,10 +148,10 @@ class MetricSemantics:  # noqa: D
 
         return tuple(input_measure_specs)
 
-    def contains_cumulative_metric(self, metric_specs: Sequence[MetricSpec]) -> bool:
+    def contains_cumulative_metric(self, metric_references: Sequence[MetricReference]) -> bool:
         """Returns true if any of the specs correspond to a cumulative metric."""
-        for metric_spec in metric_specs:
-            if self.get_metric(metric_spec).type == MetricType.CUMULATIVE:
+        for metric_reference in metric_references:
+            if self.get_metric(metric_reference).type == MetricType.CUMULATIVE:
                 return True
         return False
 
