@@ -1,6 +1,6 @@
 import logging
 
-from metricflow.model.objects.elements.measure import AggregationType
+from metricflow.aggregation_properties import AggregationType
 from metricflow.model.objects.user_configured_model import UserConfiguredModel
 from metricflow.model.transformations.transform_rule import ModelTransformRule
 
@@ -16,9 +16,9 @@ class BooleanMeasureAggregationRule(ModelTransformRule):
             for measure in data_source.measures:
                 if measure.agg == AggregationType.BOOLEAN:
                     logger.warning(
-                        f"In data source {data_source.name}, measure `{measure.name}` is configured as "
-                        f"aggregation type `boolean`, which has been deprecated. Please use `sum_boolean` "
-                        f"instead."
+                        f"In data source {data_source.name}, measure `{measure.reference.element_name}` "
+                        f"is configured as aggregation type `boolean`, which has been deprecated. Please use "
+                        f"`sum_boolean` instead."
                     )
                 if measure.agg == AggregationType.BOOLEAN or measure.agg == AggregationType.SUM_BOOLEAN:
                     if measure.expr:
