@@ -25,7 +25,7 @@ class CompositeIdentifierExpressionRule(ModelTransformRule):
                         continue
 
                     for identifier in data_source.identifiers:
-                        if sub_identifier.ref == identifier.name.element_name:
+                        if sub_identifier.ref == identifier.name:
                             sub_identifier.ref = None
                             sub_identifier.name = identifier.name
                             sub_identifier.expr = identifier.expr
