@@ -20,7 +20,7 @@ def assert_exactly_one_arg_set(**kwargs) -> None:  # type: ignore
     """Throws an assertion error if 0 or more than 1 argument is not None."""
     num_set = 0
     for value in kwargs.values():
-        if value:
+        if value is not None:
             num_set += 1
 
     assert num_set == 1, f"{num_set} argument(s) set instead of 1 in arguments: {kwargs}"
@@ -151,9 +151,22 @@ def random_id() -> str:
     return "".join(random.choices(filtered_alphabet, k=8))
 
 
-def assert_values_exhausted(value: Enum) -> NoReturn:
+def assert_values_exhausted(value: NoReturn) -> NoReturn:
     """Helper method to allow MyPy to guarantee an exhaustive switch through an enumeration or literal
 
+    DO NOT MODIFY THE TYPE SIGNATURE OF THIS FUNCTION UNLESS MYPY CHANGES HOW IT HANDLES THINGS
+
+    To use this function correctly you MUST do an exhaustive switch through ALL values, using `is` for comparison
+    (doing x == SomeEnum.VALUE will not work, nor will `x in (SomeEnum.VALUE_1, SomeEnum.VALUE_2)`).
+
+    If mypy raises an error of the form:
+      `x has incompatible type SomeEnum; expected NoReturn`
+    the switch is not constructed correctly. Fix your switch statement to use `is` for all comparisons.
+
+    If mypy raises an error of the form
+      `x has incompatible type Union[Literal...]` expected NoReturn`
+    the switch statement is non-exhaustive, and the values listed in the error message need to be accounted for.
+
     See https://mypy.readthedocs.io/en/stable/literal_types.html#exhaustiveness-checks
     For an enum example, see issue:
     https://github.com/python/mypy/issues/6366#issuecomment-560369716
