@@ -7,8 +7,8 @@ from collections import OrderedDict
 from itertools import chain
 from typing import List, Sequence, Optional, Dict, Tuple
 
+from metricflow.aggregation_properties import AggregationState
 from metricflow.instances import (
-    AggregationState,
     MdoInstance,
     DimensionInstance,
     IdentifierInstance,
@@ -18,10 +18,12 @@ from metricflow.instances import (
     InstanceSetTransform,
     TimeDimensionInstance,
 )
+from metricflow.protocols.semantics import DataSourceSemanticsAccessor
 from metricflow.object_utils import assert_exactly_one_arg_set
 from metricflow.plan_conversion.select_column_gen import SelectColumnSet
 from metricflow.specs import (
     MeasureSpec,
+    MetricInputMeasureSpec,
     InstanceSpec,
     IdentifierSpec,
     DimensionSpec,
@@ -29,6 +31,7 @@ from metricflow.specs import (
     TimeDimensionSpec,
     LinklessIdentifierSpec,
     LinkableInstanceSpec,
+    IdentifierReference,
 )
 from metricflow.sql.sql_exprs import (
     SqlColumnReferenceExpression,
@@ -36,7 +39,6 @@ from metricflow.sql.sql_exprs import (
     SqlFunctionExpression,
 )
 from metricflow.sql.sql_plan import SqlSelectColumn
-from metricflow.model.semantics.semantic_containers import DataSourceSemantics
 
 logger = logging.getLogger(__name__)
 
@@ -48,9 +50,22 @@ class CreateSelectColumnsForInstances(InstanceSetTransform[SelectColumnSet]):
     come from the given table alias.
     """
 
-    def __init__(self, table_alias: str, column_resolver: ColumnAssociationResolver) -> None:  # noqa: D
+    def __init__(
+        self,
+        table_alias: str,
+        column_resolver: ColumnAssociationResolver,
+        output_to_input_column_mapping: Optional[OrderedDict[str, str]] = None,
+    ) -> None:
+        """Initializer.
+
+        Args:
+            table_alias: the table alias to select columns from
+            column_resolver: resolver to name columns.
+            output_to_input_column_mapping: if specified, use these columns in the input for the given output columns.
+        """
         self._table_alias = table_alias
         self._column_resolver = column_resolver
+        self._output_to_input_column_mapping = output_to_input_column_mapping or OrderedDict()
 
     def transform(self, instance_set: InstanceSet) -> SelectColumnSet:  # noqa: D
         metric_cols = list(
@@ -112,13 +127,20 @@ class CreateSelectColumnsForInstances(InstanceSetTransform[SelectColumnSet]):
             f"{expected_column_associations} -- {existing_column_associations}"
         )
 
-        return [
-            SqlSelectColumn(
-                expr=SqlColumnReferenceExpression(SqlColumnReference(self._table_alias, mapped_names[0])),
-                column_alias=expected_name,
+        select_columns = []
+        for expected_name, mapped_cols in column_matches.items():
+            input_column_name = mapped_cols[0]
+            output_column_name = expected_name
+
+            if output_column_name in self._output_to_input_column_mapping:
+                input_column_name = self._output_to_input_column_mapping[output_column_name]
+            select_columns.append(
+                SqlSelectColumn(
+                    expr=SqlColumnReferenceExpression(SqlColumnReference(self._table_alias, input_column_name)),
+                    column_alias=output_column_name,
+                )
             )
-            for expected_name, mapped_names in column_matches.items()
-        ]
+        return select_columns
 
 
 class CreateSelectColumnsWithMeasuresAggregated(CreateSelectColumnsForInstances):
@@ -126,29 +148,53 @@ class CreateSelectColumnsWithMeasuresAggregated(CreateSelectColumnsForInstances)
 
     However, for measure columns, convert them into expressions like "SUM(fct_bookings.bookings) AS bookings" so that
     the resulting expressions can be used for aggregations.
+
+    Also add an output alias that conforms to the alias
     """
 
     def __init__(  # noqa: D
-        self, table_alias: str, column_resolver: ColumnAssociationResolver, data_source_semantics: DataSourceSemantics
+        self,
+        table_alias: str,
+        column_resolver: ColumnAssociationResolver,
+        data_source_semantics: DataSourceSemanticsAccessor,
+        metric_input_measure_specs: Sequence[MetricInputMeasureSpec],
     ) -> None:
         self._data_source_semantics = data_source_semantics
+        self.metric_input_measure_specs = metric_input_measure_specs
         super().__init__(table_alias=table_alias, column_resolver=column_resolver)
 
+    def _make_sql_column_expression_to_aggregate_measures(
+        self, measure_instances: Tuple[MeasureInstance, ...]
+    ) -> List[SqlSelectColumn]:
+        output_columns: List[SqlSelectColumn] = []
+        aliased_input_specs = [spec for spec in self.metric_input_measure_specs if spec.alias]
+        for instance in measure_instances:
+            matches = [spec for spec in aliased_input_specs if spec.measure_spec == instance.spec]
+            if matches:
+                aliased_spec = matches[0]
+                aliased_input_specs.remove(aliased_spec)
+                output_measure_spec = aliased_spec.post_aggregation_spec
+            else:
+                output_measure_spec = instance.spec
+
+            output_columns.append(
+                self._make_sql_column_expression_to_aggregate_measure(
+                    measure_instance=instance, output_measure_spec=output_measure_spec
+                )
+            )
+
+        return output_columns
+
     def _make_sql_column_expression_to_aggregate_measure(  # noqa: D
-        self, measure_instance: MeasureInstance
+        self, measure_instance: MeasureInstance, output_measure_spec: MeasureSpec
     ) -> SqlSelectColumn:
         """Convert one measure instance into a SQL column"""
         # Get the column name of the measure in the table that we're reading from
         column_name_in_table = measure_instance.associated_column.column_name
 
         # Create an expression that will aggregate the given measure.
-        new_column_association_for_aggregated_measure = self._column_resolver.resolve_measure_spec(
-            measure_instance.spec
-        )
-        new_column_name_for_aggregated_measure = new_column_association_for_aggregated_measure.column_name
-
         # Figure out the aggregation function for the measure.
-        measure = self._data_source_semantics.get_measure(measure_instance.spec)
+        measure = self._data_source_semantics.get_measure(measure_instance.spec.as_reference)
         aggregation_type = measure.agg
 
         expression_to_get_measure = SqlColumnReferenceExpression(
@@ -159,6 +205,11 @@ class CreateSelectColumnsWithMeasuresAggregated(CreateSelectColumnsForInstances)
             aggregation_type=aggregation_type, sql_column_expression=expression_to_get_measure
         )
 
+        # Get the output column name from the measure/alias
+
+        new_column_association_for_aggregated_measure = self._column_resolver.resolve_measure_spec(output_measure_spec)
+        new_column_name_for_aggregated_measure = new_column_association_for_aggregated_measure.column_name
+
         return SqlSelectColumn(
             expr=expression_to_aggregate_measure,
             column_alias=new_column_name_for_aggregated_measure,
@@ -168,9 +219,8 @@ class CreateSelectColumnsWithMeasuresAggregated(CreateSelectColumnsForInstances)
         metric_cols = list(
             chain.from_iterable([self._make_sql_column_expression(x) for x in instance_set.metric_instances])
         )
-        measure_cols = [
-            self._make_sql_column_expression_to_aggregate_measure(x) for x in instance_set.measure_instances
-        ]
+
+        measure_cols = self._make_sql_column_expression_to_aggregate_measures(instance_set.measure_instances)
         dimension_cols = list(
             chain.from_iterable([self._make_sql_column_expression(x) for x in instance_set.dimension_instances])
         )
@@ -224,7 +274,10 @@ class AddLinkToLinkableElements(InstanceSetTransform[InstanceSet]):
             # The new dimension spec should include the join on identifier.
             transformed_time_dimension_spec_from_right = TimeDimensionSpec(
                 element_name=time_dimension_instance.spec.element_name,
-                identifier_links=(self._join_on_identifier,) + time_dimension_instance.spec.identifier_links,
+                identifier_links=(
+                    (IdentifierReference(element_name=self._join_on_identifier.element_name),)
+                    + time_dimension_instance.spec.identifier_links
+                ),
                 time_granularity=time_dimension_instance.spec.time_granularity,
             )
             time_dimension_instances_with_additional_link.append(
@@ -283,7 +336,10 @@ class FilterLinkableInstancesWithLeadingLink(InstanceSetTransform[InstanceSet]):
         self._identifier_link = identifier_link
 
     def _should_pass(self, linkable_spec: LinkableInstanceSpec) -> bool:  # noqa: D
-        return len(linkable_spec.identifier_links) == 0 or linkable_spec.identifier_links[0] != self._identifier_link
+        return (
+            len(linkable_spec.identifier_links) == 0
+            or LinklessIdentifierSpec.from_reference(linkable_spec.identifier_links[0]) != self._identifier_link
+        )
 
     def transform(self, instance_set: InstanceSet) -> InstanceSet:  # noqa: D
 
@@ -385,11 +441,8 @@ class ChangeMeasureAggregationState(InstanceSetTransform[InstanceSet]):
             MeasureInstance(
                 associated_columns=x.associated_columns,
                 defined_from=x.defined_from,
-                source_time_dimension_reference=x.source_time_dimension_reference,
                 aggregation_state=self._aggregation_state_changes[x.aggregation_state],
-                spec=MeasureSpec(
-                    element_name=x.spec.element_name,
-                ),
+                spec=x.spec,
             )
             for x in instance_set.measure_instances
         )
@@ -402,6 +455,59 @@ class ChangeMeasureAggregationState(InstanceSetTransform[InstanceSet]):
         )
 
 
+class AliasAggregatedMeasures(InstanceSetTransform[InstanceSet]):
+    """Returns a new instance set where all measures have been assigned an alias spec"""
+
+    def __init__(self, metric_input_measure_specs: Sequence[MetricInputMeasureSpec]):
+        """Initializer stores the input specs, which contain the aliases for each measure
+
+        Note this class only works if used in conjunction with an AggregateMeasuresNode that has been generated
+        by querying a single data source for a single set of aggregated measures. This is currently enforced
+        by the structure of the DataflowPlanBuilder, which ensures each AggregateMeasuresNode corresponds to
+        a single data source set of measures for a single metric, and that these outputs will then be
+        combinded via joins.
+        """
+        self.metric_input_measure_specs = metric_input_measure_specs
+
+    def _alias_measure_instances(self, measure_instances: Tuple[MeasureInstance, ...]) -> Tuple[MeasureInstance, ...]:
+        """Update all measure instances with aliases, if any are found in the input spec set"""
+        aliased_instances: List[MeasureInstance] = []
+        aliased_input_specs = [spec for spec in self.metric_input_measure_specs if spec.alias]
+        for instance in measure_instances:
+            matches = [spec for spec in aliased_input_specs if spec.measure_spec == instance.spec]
+            assert (
+                len(matches) < 2
+            ), f"Found duplicate aliased measure spec matches: {matches} for measure instance {instance}. "
+            "We should always have 0 or 1 matches, or else we might pass the wrong aggregated measures to the "
+            "downstream metric computation expression!"
+            if matches:
+                aliased_spec = matches[0]
+                aliased_input_specs.remove(aliased_spec)
+                measure_spec = aliased_spec.post_aggregation_spec
+            else:
+                measure_spec = instance.spec
+
+            aliased_instances.append(
+                MeasureInstance(
+                    associated_columns=instance.associated_columns,
+                    spec=measure_spec,
+                    aggregation_state=instance.aggregation_state,
+                    defined_from=instance.defined_from,
+                )
+            )
+
+        return tuple(aliased_instances)
+
+    def transform(self, instance_set: InstanceSet) -> InstanceSet:  # noqa: D
+        return InstanceSet(
+            measure_instances=self._alias_measure_instances(instance_set.measure_instances),
+            dimension_instances=instance_set.dimension_instances,
+            time_dimension_instances=instance_set.time_dimension_instances,
+            identifier_instances=instance_set.identifier_instances,
+            metric_instances=instance_set.metric_instances,
+        )
+
+
 class AddMetrics(InstanceSetTransform[InstanceSet]):
     """Adds the given metric instances to the instance set"""
 
@@ -449,7 +555,6 @@ class ChangeAssociatedColumns(InstanceSetTransform[InstanceSet]):
                     ),
                     spec=input_measure_instance.spec,
                     defined_from=input_measure_instance.defined_from,
-                    source_time_dimension_reference=input_measure_instance.source_time_dimension_reference,
                     aggregation_state=input_measure_instance.aggregation_state,
                 )
             )
