@@ -5,25 +5,24 @@ from _pytest.fixtures import FixtureRequest
 
 from metricflow.dataflow.builder.dataflow_plan_builder import DataflowPlanBuilder
 from metricflow.dataflow.dataflow_plan_to_text import dataflow_plan_as_text
+from metricflow.dataset.data_source_adapter import DataSourceDataSet
 from metricflow.errors.errors import UnableToSatisfyQueryError
 from metricflow.specs import (
     MetricFlowQuerySpec,
     MetricSpec,
     DimensionSpec,
-    LinklessIdentifierSpec,
-    TimeDimensionSpec,
     SpecWhereClauseConstraint,
     LinkableSpecSet,
+    IdentifierReference,
 )
 from metricflow.specs import (
     OrderBySpec,
 )
 from metricflow.sql.sql_bind_parameters import SqlBindParameters
-from metricflow.time.time_granularity import TimeGranularity
-from metricflow.dataset.data_source_adapter import DataSourceDataSet
-from metricflow.test.fixtures.setup_fixtures import MetricFlowTestSessionState
 from metricflow.test.dataflow_plan_to_svg import display_graph_if_requested
+from metricflow.test.fixtures.setup_fixtures import MetricFlowTestSessionState
 from metricflow.test.plan_utils import assert_plan_snapshot_text_equal
+from metricflow.test.time.metric_time_dimension import MTD_SPEC_DAY, MTD, MTD_SPEC_MONTH
 
 logger = logging.getLogger(__name__)
 
@@ -76,7 +75,7 @@ def test_joined_plan(  # noqa: D
                 ),
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
         )
@@ -105,22 +104,14 @@ def test_order_by_plan(  # noqa: D
     dataflow_plan = dataflow_plan_builder.build_plan(
         MetricFlowQuerySpec(
             metric_specs=(MetricSpec(element_name="bookings"),),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
-                ),
-            ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
             order_by_specs=(
                 OrderBySpec(
-                    item=TimeDimensionSpec(
-                        element_name="ds",
-                        identifier_links=(),
-                    ),
+                    time_dimension_spec=MTD_SPEC_DAY,
                     descending=False,
                 ),
                 OrderBySpec(
-                    item=MetricSpec(element_name="bookings"),
+                    metric_spec=MetricSpec(element_name="bookings"),
                     descending=True,
                 ),
             ),
@@ -150,12 +141,7 @@ def test_limit_rows_plan(  # noqa: D
     dataflow_plan = dataflow_plan_builder.build_plan(
         MetricFlowQuerySpec(
             metric_specs=(MetricSpec(element_name="bookings"),),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
-                ),
-            ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
             limit=1,
         )
     )
@@ -189,12 +175,7 @@ def test_multiple_metrics_plan(  # noqa: D
                     identifier_links=(),
                 ),
             ),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
-                ),
-            ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
         )
     )
 
@@ -224,15 +205,10 @@ def test_expr_metrics_plan(
             dimension_specs=(
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name(element_name="listing"),),
-                ),
-            ),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
         )
     )
 
@@ -262,15 +238,10 @@ def test_single_data_source_ratio_metrics_plan(
             dimension_specs=(
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name(element_name="listing"),),
-                ),
-            ),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
         )
     )
 
@@ -300,15 +271,10 @@ def test_multi_data_source_ratio_metrics_plan(
             dimension_specs=(
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name(element_name="listing"),),
-                ),
-            ),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
+            time_dimension_specs=(MTD_SPEC_DAY,),
         )
     )
 
@@ -339,8 +305,8 @@ def test_multihop_join_plan(  # noqa: D
                 DimensionSpec(
                     element_name="customer_name",
                     identifier_links=(
-                        LinklessIdentifierSpec.from_element_name(element_name="account_id"),
-                        LinklessIdentifierSpec.from_element_name(element_name="customer_id"),
+                        IdentifierReference(element_name="account_id"),
+                        IdentifierReference(element_name="customer_id"),
                     ),
                 ),
             ),
@@ -383,7 +349,7 @@ def test_where_constrained_plan(  # noqa: D
                     dimension_specs=(
                         DimensionSpec(
                             element_name="country_latest",
-                            identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+                            identifier_links=(IdentifierReference(element_name="listing"),),
                         ),
                     )
                 ),
@@ -422,16 +388,9 @@ def test_where_constrained_plan_time_dimension(  # noqa: D
                 ),
             ),
             where_constraint=SpecWhereClauseConstraint(
-                where_condition="ds >= '2020-01-01'",
-                linkable_names=("ds",),
-                linkable_spec_set=LinkableSpecSet(
-                    time_dimension_specs=(
-                        TimeDimensionSpec(
-                            element_name="ds",
-                            identifier_links=(),
-                        ),
-                    )
-                ),
+                where_condition=f"{MTD} >= '2020-01-01'",
+                linkable_names=(MTD,),
+                linkable_spec_set=LinkableSpecSet(time_dimension_specs=(MTD_SPEC_DAY,)),
                 execution_parameters=SqlBindParameters(),
             ),
         )
@@ -463,7 +422,7 @@ def test_where_constrained_with_common_linkable_plan(  # noqa: D
             dimension_specs=(
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
             where_constraint=SpecWhereClauseConstraint(
@@ -473,7 +432,7 @@ def test_where_constrained_with_common_linkable_plan(  # noqa: D
                     dimension_specs=(
                         DimensionSpec(
                             element_name="country_latest",
-                            identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+                            identifier_links=(IdentifierReference(element_name="listing"),),
                         ),
                     )
                 ),
@@ -509,8 +468,8 @@ def test_multihop_join_plan_ambiguous_dim(  # noqa: D
                     DimensionSpec(
                         element_name="home_country",
                         identifier_links=(
-                            LinklessIdentifierSpec.from_element_name(element_name="listing"),
-                            LinklessIdentifierSpec.from_element_name(element_name="user"),
+                            IdentifierReference(element_name="listing"),
+                            IdentifierReference(element_name="user"),
                         ),
                     ),
                 ),
@@ -528,13 +487,7 @@ def test_cumulative_metric(  # noqa: D
         MetricFlowQuerySpec(
             metric_specs=(MetricSpec(element_name="trailing_2_months_revenue"),),
             dimension_specs=(),
-            time_dimension_specs=(
-                TimeDimensionSpec(
-                    element_name="ds",
-                    identifier_links=(),
-                    time_granularity=TimeGranularity.MONTH,
-                ),
-            ),
+            time_dimension_specs=(MTD_SPEC_MONTH,),
         )
     )
 
@@ -562,7 +515,7 @@ def test_distinct_values_plan(  # noqa: D
         metric_specs=(MetricSpec(element_name="bookings"),),
         dimension_spec=DimensionSpec(
             element_name="country_latest",
-            identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+            identifier_links=(IdentifierReference(element_name="listing"),),
         ),
         limit=100,
     )
@@ -579,3 +532,59 @@ def test_distinct_values_plan(  # noqa: D
         mf_test_session_state=mf_test_session_state,
         dag_graph=dataflow_plan,
     )
+
+
+def test_measure_constraint_plan(
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
+) -> None:
+    """Tests a plan for querying a metric with a constraint on one or more of its input measures."""
+    dataflow_plan = dataflow_plan_builder.build_plan(
+        MetricFlowQuerySpec(
+            metric_specs=(MetricSpec(element_name="lux_booking_value_rate_expr"),),
+            dimension_specs=(),
+            time_dimension_specs=(MTD_SPEC_DAY,),
+        ),
+    )
+
+    assert_plan_snapshot_text_equal(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        plan=dataflow_plan,
+        plan_snapshot_text=dataflow_plan_as_text(dataflow_plan),
+    )
+
+    display_graph_if_requested(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dag_graph=dataflow_plan,
+    )
+
+
+def test_measure_constraint_with_reused_measure_plan(
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
+) -> None:
+    """Tests a plan for querying a metric with a constraint on one or more of its input measures."""
+    dataflow_plan = dataflow_plan_builder.build_plan(
+        MetricFlowQuerySpec(
+            metric_specs=(MetricSpec(element_name="instant_booking_value_ratio"),),
+            dimension_specs=(),
+            time_dimension_specs=(MTD_SPEC_DAY,),
+        ),
+    )
+
+    assert_plan_snapshot_text_equal(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        plan=dataflow_plan,
+        plan_snapshot_text=dataflow_plan_as_text(dataflow_plan),
+    )
+
+    display_graph_if_requested(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dag_graph=dataflow_plan,
+    )
