@@ -7,7 +7,7 @@ from metricflow.model.semantics.data_source_container import PydanticDataSourceC
 from metricflow.model.semantics.linkable_spec_resolver import LinkableElementProperties
 from metricflow.model.semantics.semantic_containers import DataSourceSemantics, MetricSemantics
 from metricflow.references import IdentifierReference, MeasureReference
-from metricflow.specs import MetricSpec
+from metricflow.references import MetricReference
 
 logger = logging.getLogger(__name__)
 
@@ -113,7 +113,7 @@ def test_elements_for_metric(new_metric_semantics: MetricSemantics) -> None:  #
         [
             x.qualified_name
             for x in new_metric_semantics.element_specs_for_metrics(
-                [MetricSpec(element_name="views")],
+                [MetricReference(element_name="views")],
                 without_any_property=frozenset({LinkableElementProperties.DERIVED_TIME_GRANULARITY}),
             )
         ]
@@ -157,7 +157,7 @@ def test_elements_for_metric(new_metric_semantics: MetricSemantics) -> None:  #
     }
 
     local_specs = new_metric_semantics.element_specs_for_metrics(
-        metric_specs=[MetricSpec(element_name="views")],
+        metric_references=[MetricReference(element_name="views")],
         with_any_property=frozenset({LinkableElementProperties.LOCAL}),
         without_any_property=frozenset({LinkableElementProperties.DERIVED_TIME_GRANULARITY}),
     )
@@ -175,7 +175,7 @@ def test_local_linked_elements_for_metric(new_metric_semantics: MetricSemantics)
         [
             x.qualified_name
             for x in new_metric_semantics.element_specs_for_metrics(
-                [MetricSpec(element_name="listings")],
+                [MetricReference(element_name="listings")],
                 with_any_property=frozenset({LinkableElementProperties.LOCAL_LINKED}),
                 without_any_property=frozenset({LinkableElementProperties.DERIVED_TIME_GRANULARITY}),
             )
