@@ -17,11 +17,15 @@ def test_lonely_identifier_raises_issue(simple_model__pre_transforms: UserConfig
 
     func: Callable[[DataSource], bool] = lambda data_source: len(data_source.identifiers) > 0
     data_source_with_identifiers, _ = find_data_source_with(model, func)
-    data_source_with_identifiers.identifiers[0].name = IdentifierSpec.parse(lonely_identifier_name)
-    build = ModelValidator.validate_model(model)
+    data_source_with_identifiers.identifiers[0].name = IdentifierSpec.from_name(lonely_identifier_name).element_name
+    model_validator = ModelValidator()
+    build = model_validator.validate_model(model)
 
     found_warning = False
-    warning = f"Identifier `{lonely_identifier_name}` only found in one data source `{data_source_with_identifiers.name}` which means it will be unused in joins."
+    warning = (
+        f"Identifier `{lonely_identifier_name}` only found in one data source `{data_source_with_identifiers.name}` "
+        f"which means it will be unused in joins."
+    )
     if build.issues is not None:
         for issue in build.issues:
             if re.search(warning, issue.message):
