@@ -3,6 +3,7 @@ from typing import List
 import pytest
 from _pytest.fixtures import FixtureRequest
 
+from metricflow.aggregation_properties import AggregationType
 from metricflow.constraints.time_constraint import TimeRangeConstraint
 from metricflow.dataflow.builder.dataflow_plan_builder import DataflowPlanBuilder
 from metricflow.dataflow.dataflow_plan import (
@@ -17,6 +18,8 @@ from metricflow.dataflow.dataflow_plan import (
     OrderByLimitNode,
     ConstrainTimeRangeNode,
     BaseOutput,
+    MetricTimeDimensionTransformNode,
+    SemiAdditiveJoinNode,
 )
 from metricflow.dataflow.dataflow_plan_to_text import dataflow_plan_as_text
 from metricflow.model.semantic_model import SemanticModel
@@ -24,14 +27,16 @@ from metricflow.plan_conversion.column_resolver import DefaultColumnAssociationR
 from metricflow.plan_conversion.dataflow_to_sql import DataflowToSqlQueryPlanConverter
 from metricflow.plan_conversion.time_spine import TimeSpineSource
 from metricflow.protocols.sql_client import SqlClient
+from metricflow.references import TimeDimensionReference, IdentifierReference
 from metricflow.specs import (
     DimensionSpec,
     IdentifierSpec,
-    InstanceSpec,
     MeasureSpec,
+    MetricInputMeasureSpec,
     MetricSpec,
     LinklessIdentifierSpec,
     MetricFlowQuerySpec,
+    NonAdditiveDimensionSpec,
     OrderBySpec,
     TimeDimensionSpec,
     SpecWhereClauseConstraint,
@@ -39,6 +44,7 @@ from metricflow.specs import (
 )
 from metricflow.sql.optimizer.optimization_levels import SqlQueryOptimizationLevel
 from metricflow.sql.sql_bind_parameters import SqlBindParameters
+from metricflow.test.time.metric_time_dimension import MTD_SPEC_DAY
 from metricflow.time.time_granularity import TimeGranularity
 from metricflow.dataset.data_source_adapter import DataSourceDataSet
 from metricflow.test.fixtures.setup_fixtures import MetricFlowTestSessionState
@@ -62,18 +68,6 @@ def composite_dataflow_to_sql_converter(  # noqa: D
     )
 
 
-@pytest.fixture(scope="session")
-def dataflow_to_sql_converter(  # noqa: D
-    simple_semantic_model: SemanticModel,
-    time_spine_source: TimeSpineSource,
-) -> DataflowToSqlQueryPlanConverter[DataSourceDataSet]:
-    return DataflowToSqlQueryPlanConverter[DataSourceDataSet](
-        column_association_resolver=DefaultColumnAssociationResolver(simple_semantic_model),
-        semantic_model=simple_semantic_model,
-        time_spine_source=time_spine_source,
-    )
-
-
 @pytest.fixture(scope="session")
 def multihop_dataflow_to_sql_converter(  # noqa: D
     multi_hop_join_semantic_model: SemanticModel,
@@ -251,7 +245,8 @@ def test_measure_aggregation_node(  # noqa: D
     count_distinct_spec = MeasureSpec(
         element_name="bookers",
     )
-    measure_specs: List[InstanceSpec] = [sum_spec, sum_boolean_spec, avg_spec, count_distinct_spec]
+    measure_specs: List[MeasureSpec] = [sum_spec, sum_boolean_spec, avg_spec, count_distinct_spec]
+    metric_input_measure_specs = tuple(MetricInputMeasureSpec(measure_spec=x) for x in measure_specs)
 
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode[DataSourceDataSet](
@@ -259,7 +254,9 @@ def test_measure_aggregation_node(  # noqa: D
         include_specs=measure_specs,
     )
 
-    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](filtered_measure_node)
+    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](
+        parent_node=filtered_measure_node, metric_input_measure_specs=metric_input_measure_specs
+    )
 
     convert_and_check(
         request=request,
@@ -281,7 +278,7 @@ def test_single_join_node(  # noqa: D
     measure_spec = MeasureSpec(
         element_name="bookings",
     )
-    identifier_spec = LinklessIdentifierSpec(element_name="listing", identifier_links=())
+    identifier_spec = LinklessIdentifierSpec.from_element_name(element_name="listing")
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode[DataSourceDataSet](
         parent_node=measure_source_node, include_specs=[measure_spec, identifier_spec]
@@ -328,7 +325,7 @@ def test_multi_join_node(
     measure_spec = MeasureSpec(
         element_name="bookings",
     )
-    identifier_spec = LinklessIdentifierSpec(element_name="listing", identifier_links=())
+    identifier_spec = LinklessIdentifierSpec.from_element_name(element_name="listing")
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode[DataSourceDataSet](
         parent_node=measure_source_node, include_specs=[measure_spec, identifier_spec]
@@ -348,13 +345,13 @@ def test_multi_join_node(
         join_targets=[
             JoinDescription(
                 join_node=filtered_dimension_node,
-                join_on_identifier=LinklessIdentifierSpec.from_element_name("listing"),
+                join_on_identifier=LinklessIdentifierSpec.from_element_name(element_name="listing"),
                 join_on_partition_dimensions=(),
                 join_on_partition_time_dimensions=(),
             ),
             JoinDescription(
                 join_node=filtered_dimension_node,
-                join_on_identifier=LinklessIdentifierSpec.from_element_name("listing"),
+                join_on_identifier=LinklessIdentifierSpec.from_element_name(element_name="listing"),
                 join_on_partition_dimensions=(),
                 join_on_partition_time_dimensions=(),
             ),
@@ -381,7 +378,8 @@ def test_compute_metrics_node(
     measure_spec = MeasureSpec(
         element_name="bookings",
     )
-    identifier_spec = LinklessIdentifierSpec(element_name="listing", identifier_links=())
+    identifier_spec = LinklessIdentifierSpec.from_element_name(element_name="listing")
+    metric_input_measure_specs = (MetricInputMeasureSpec(measure_spec=measure_spec),)
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode[DataSourceDataSet](
         parent_node=measure_source_node, include_specs=[measure_spec, identifier_spec]
@@ -408,7 +406,9 @@ def test_compute_metrics_node(
         ],
     )
 
-    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](join_node)
+    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](
+        parent_node=join_node, metric_input_measure_specs=metric_input_measure_specs
+    )
 
     metric_spec = MetricSpec(element_name="bookings")
     compute_metrics_node = ComputeMetricsNode[DataSourceDataSet](
@@ -435,7 +435,8 @@ def test_compute_metrics_node_simple_expr(
     measure_spec = MeasureSpec(
         element_name="booking_value",
     )
-    identifier_spec = LinklessIdentifierSpec(element_name="listing", identifier_links=())
+    identifier_spec = LinklessIdentifierSpec.from_element_name(element_name="listing")
+    metric_input_measure_specs = (MetricInputMeasureSpec(measure_spec=measure_spec),)
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode[DataSourceDataSet](
         parent_node=measure_source_node, include_specs=[measure_spec, identifier_spec]
@@ -462,7 +463,9 @@ def test_compute_metrics_node_simple_expr(
         ],
     )
 
-    aggregated_measures_node = AggregateMeasuresNode[DataSourceDataSet](join_node)
+    aggregated_measures_node = AggregateMeasuresNode[DataSourceDataSet](
+        parent_node=join_node, metric_input_measure_specs=metric_input_measure_specs
+    )
     metric_spec = MetricSpec(element_name="booking_fees")
     compute_metrics_node = ComputeMetricsNode[DataSourceDataSet](
         parent_node=aggregated_measures_node, metric_specs=[metric_spec]
@@ -507,7 +510,11 @@ def test_compute_metrics_node_ratio_from_single_data_source(
     denominator_spec = MeasureSpec(
         element_name="bookers",
     )
-    identifier_spec = LinklessIdentifierSpec(element_name="listing", identifier_links=())
+    identifier_spec = LinklessIdentifierSpec.from_element_name(element_name="listing")
+    metric_input_measure_specs = (
+        MetricInputMeasureSpec(measure_spec=numerator_spec),
+        MetricInputMeasureSpec(measure_spec=denominator_spec),
+    )
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measures_node = FilterElementsNode[DataSourceDataSet](
         parent_node=measure_source_node, include_specs=[numerator_spec, denominator_spec, identifier_spec]
@@ -534,7 +541,9 @@ def test_compute_metrics_node_ratio_from_single_data_source(
         ],
     )
 
-    aggregated_measures_node = AggregateMeasuresNode[DataSourceDataSet](join_node)
+    aggregated_measures_node = AggregateMeasuresNode[DataSourceDataSet](
+        parent_node=join_node, metric_input_measure_specs=metric_input_measure_specs
+    )
     metric_spec = MetricSpec(element_name="bookings_per_booker")
     compute_metrics_node = ComputeMetricsNode[DataSourceDataSet](
         parent_node=aggregated_measures_node, metric_specs=[metric_spec]
@@ -563,7 +572,7 @@ def test_compute_metrics_node_ratio_from_multiple_data_sources(
     """
     dimension_spec = DimensionSpec(
         element_name="country_latest",
-        identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+        identifier_links=(IdentifierReference(element_name="listing"),),
     )
     time_dimension_spec = TimeDimensionSpec(
         element_name="ds",
@@ -599,6 +608,7 @@ def test_order_by_node(
     measure_spec = MeasureSpec(
         element_name="bookings",
     )
+    metric_input_measure_specs = (MetricInputMeasureSpec(measure_spec=measure_spec),)
 
     dimension_spec = DimensionSpec(
         element_name="is_instant",
@@ -615,7 +625,9 @@ def test_order_by_node(
         parent_node=measure_source_node, include_specs=[measure_spec, dimension_spec, time_dimension_spec]
     )
 
-    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](filtered_measure_node)
+    aggregated_measure_node = AggregateMeasuresNode[DataSourceDataSet](
+        parent_node=filtered_measure_node, metric_input_measure_specs=metric_input_measure_specs
+    )
 
     metric_spec = MetricSpec(element_name="bookings")
     compute_metrics_node = ComputeMetricsNode[DataSourceDataSet](
@@ -625,11 +637,11 @@ def test_order_by_node(
     order_by_node = OrderByLimitNode(
         order_by_specs=[
             OrderBySpec(
-                item=time_dimension_spec,
+                time_dimension_spec=time_dimension_spec,
                 descending=False,
             ),
             OrderBySpec(
-                item=metric_spec,
+                metric_spec=metric_spec,
                 descending=True,
             ),
         ],
@@ -660,8 +672,8 @@ def test_multihop_node(
                 DimensionSpec(
                     element_name="customer_name",
                     identifier_links=(
-                        LinklessIdentifierSpec.from_element_name(element_name="account_id"),
-                        LinklessIdentifierSpec.from_element_name(element_name="customer_id"),
+                        IdentifierReference(element_name="account_id"),
+                        IdentifierReference(element_name="customer_id"),
                     ),
                 ),
             ),
@@ -702,7 +714,7 @@ def test_filter_with_where_constraint_on_join_dim(
                     dimension_specs=(
                         DimensionSpec(
                             element_name="country_latest",
-                            identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+                            identifier_links=(IdentifierReference(element_name="listing"),),
                         ),
                     )
                 ),
@@ -720,14 +732,14 @@ def test_filter_with_where_constraint_on_join_dim(
     )
 
 
-def test_constrain_primary_time_dimension(
+def test_constrain_time_range_node(
     request: FixtureRequest,
     mf_test_session_state: MetricFlowTestSessionState,
     consistent_id_object_repository: ConsistentIdObjectRepository,
     dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
     sql_client: SqlClient,
 ) -> None:
-    """Tests converting a dataflow plan to an SQL query plan there is a leaf compute metrics node."""
+    """Tests converting the ConstrainTimeRangeNode to SQL."""
 
     measure_source_node = consistent_id_object_repository.simple_model_read_nodes["bookings_source"]
     filtered_measure_node = FilterElementsNode(
@@ -739,8 +751,13 @@ def test_constrain_primary_time_dimension(
             TimeDimensionSpec(element_name="ds", identifier_links=(), time_granularity=TimeGranularity.DAY),
         ],
     )
-    constrain_time_node = ConstrainTimeRangeNode[DataSourceDataSet](
+    metric_time_node = MetricTimeDimensionTransformNode(
         parent_node=filtered_measure_node,
+        aggregation_time_dimension_reference=TimeDimensionReference(element_name="ds"),
+    )
+
+    constrain_time_node = ConstrainTimeRangeNode[DataSourceDataSet](
+        parent_node=metric_time_node,
         time_range_constraint=TimeRangeConstraint(
             start_time=as_datetime("2020-01-01"),
             end_time=as_datetime("2020-01-02"),
@@ -963,7 +980,7 @@ def test_partitioned_join(
             dimension_specs=(
                 DimensionSpec(
                     element_name="home_state",
-                    identifier_links=(LinklessIdentifierSpec.from_element_name("user"),),
+                    identifier_links=(IdentifierReference(element_name="user"),),
                 ),
             ),
         )
@@ -1043,7 +1060,9 @@ def test_composite_identifier_with_order_by(  # noqa: D
             metric_specs=(MetricSpec(element_name="messages"),),
             identifier_specs=(IdentifierSpec(element_name="user_team", identifier_links=()),),
             order_by_specs=(
-                OrderBySpec(item=IdentifierSpec(element_name="user_team", identifier_links=()), descending=True),
+                OrderBySpec(
+                    identifier_spec=IdentifierSpec(element_name="user_team", identifier_links=()), descending=True
+                ),
             ),
         )
     )
@@ -1070,7 +1089,7 @@ def test_composite_identifier_with_join(  # noqa: D
             dimension_specs=(
                 DimensionSpec(
                     element_name="country",
-                    identifier_links=(LinklessIdentifierSpec(element_name="user_team", identifier_links=()),),
+                    identifier_links=(IdentifierReference(element_name="user_team"),),
                 ),
             ),
             identifier_specs=(IdentifierSpec(element_name="user_team", identifier_links=()),),
@@ -1098,7 +1117,7 @@ def test_distinct_values(  # noqa: D
         metric_specs=(MetricSpec(element_name="bookings"),),
         dimension_spec=DimensionSpec(
             element_name="country_latest",
-            identifier_links=(LinklessIdentifierSpec.from_element_name("listing"),),
+            identifier_links=(IdentifierReference(element_name="listing"),),
         ),
         limit=100,
     )
@@ -1125,7 +1144,7 @@ def test_local_dimension_using_local_identifier(  # noqa: D
             dimension_specs=(
                 DimensionSpec(
                     element_name="country_latest",
-                    identifier_links=(LinklessIdentifierSpec(element_name="listing", identifier_links=()),),
+                    identifier_links=(IdentifierReference(element_name="listing"),),
                 ),
             ),
         )
@@ -1138,3 +1157,163 @@ def test_local_dimension_using_local_identifier(  # noqa: D
         sql_client=sql_client,
         node=dataflow_plan.sink_output_nodes[0].parent_node,
     )
+
+
+def test_semi_additive_join_node(
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    consistent_id_object_repository: ConsistentIdObjectRepository,
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    """Tests converting a dataflow plan to a SQL query plan using a SemiAdditiveJoinNode."""
+    non_additive_dimension_spec = NonAdditiveDimensionSpec(name="ds", window_choice=AggregationType.MIN)
+    time_dimension_spec = TimeDimensionSpec(element_name="ds", identifier_links=())
+
+    measure_source_node = consistent_id_object_repository.simple_model_read_nodes["accounts_source"]
+    semi_additive_join_node = SemiAdditiveJoinNode[DataSourceDataSet](
+        parent_node=measure_source_node,
+        identifier_specs=tuple(),
+        time_dimension_spec=time_dimension_spec,
+        agg_by_function=non_additive_dimension_spec.window_choice,
+    )
+
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=semi_additive_join_node,
+    )
+
+
+def test_semi_additive_join_node_with_queried_group_by(
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    consistent_id_object_repository: ConsistentIdObjectRepository,
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    """Tests converting a dataflow plan to a SQL query plan using a SemiAdditiveJoinNode."""
+    non_additive_dimension_spec = NonAdditiveDimensionSpec(name="ds", window_choice=AggregationType.MIN)
+    time_dimension_spec = TimeDimensionSpec(element_name="ds", identifier_links=())
+    queried_time_dimension_spec = TimeDimensionSpec(
+        element_name="ds", identifier_links=(), time_granularity=TimeGranularity.WEEK
+    )
+
+    measure_source_node = consistent_id_object_repository.simple_model_read_nodes["accounts_source"]
+    semi_additive_join_node = SemiAdditiveJoinNode[DataSourceDataSet](
+        parent_node=measure_source_node,
+        identifier_specs=tuple(),
+        time_dimension_spec=time_dimension_spec,
+        agg_by_function=non_additive_dimension_spec.window_choice,
+        queried_time_dimension_spec=queried_time_dimension_spec,
+    )
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=semi_additive_join_node,
+    )
+
+
+def test_semi_additive_join_node_with_grouping(
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    consistent_id_object_repository: ConsistentIdObjectRepository,
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    """Tests converting a dataflow plan to a SQL query plan using a SemiAdditiveJoinNode with a window_grouping."""
+    non_additive_dimension_spec = NonAdditiveDimensionSpec(
+        name="ds",
+        window_choice=AggregationType.MAX,
+        window_groupings=("user",),
+    )
+    identifier_spec = LinklessIdentifierSpec(element_name="user", identifier_links=())
+    time_dimension_spec = TimeDimensionSpec(element_name="ds", identifier_links=())
+
+    measure_source_node = consistent_id_object_repository.simple_model_read_nodes["accounts_source"]
+    semi_additive_join_node = SemiAdditiveJoinNode[DataSourceDataSet](
+        parent_node=measure_source_node,
+        identifier_specs=(identifier_spec,),
+        time_dimension_spec=time_dimension_spec,
+        agg_by_function=non_additive_dimension_spec.window_choice,
+    )
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=semi_additive_join_node,
+    )
+
+
+def test_measure_constraint(  # noqa: D
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    dataflow_plan = dataflow_plan_builder.build_plan(
+        MetricFlowQuerySpec(
+            metric_specs=(MetricSpec(element_name="lux_booking_value_rate_expr"),),
+            time_dimension_specs=(MTD_SPEC_DAY,),
+        )
+    )
+
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=dataflow_plan.sink_output_nodes[0].parent_node,
+    )
+
+
+def test_measure_constraint_with_reused_measure(  # noqa: D
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    dataflow_plan = dataflow_plan_builder.build_plan(
+        MetricFlowQuerySpec(
+            metric_specs=(MetricSpec(element_name="instant_booking_value_ratio"),),
+            time_dimension_specs=(MTD_SPEC_DAY,),
+        )
+    )
+
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=dataflow_plan.sink_output_nodes[0].parent_node,
+    )
+
+
+def test_measure_constraint_with_single_expr_and_alias(  # noqa: D
+    request: FixtureRequest,
+    mf_test_session_state: MetricFlowTestSessionState,
+    dataflow_plan_builder: DataflowPlanBuilder[DataSourceDataSet],
+    dataflow_to_sql_converter: DataflowToSqlQueryPlanConverter[DataSourceDataSet],
+    sql_client: SqlClient,
+) -> None:
+    dataflow_plan = dataflow_plan_builder.build_plan(
+        MetricFlowQuerySpec(
+            metric_specs=(MetricSpec(element_name="double_counted_delayed_bookings"),),
+            time_dimension_specs=(MTD_SPEC_DAY,),
+        )
+    )
+
+    convert_and_check(
+        request=request,
+        mf_test_session_state=mf_test_session_state,
+        dataflow_to_sql_converter=dataflow_to_sql_converter,
+        sql_client=sql_client,
+        node=dataflow_plan.sink_output_nodes[0].parent_node,
+    )
