@@ -7,6 +7,8 @@ SELECT
   , booking_value AS min_booking_value
   , guest_id AS bookers
   , booking_value AS average_booking_value
+  , booking_value AS booking_payments
+  , CASE WHEN referrer_id IS NOT NULL THEN 1 ELSE 0 END AS referred_bookings
   , is_instant
   , ds
   , DATE_TRUNC(ds, isoweek) AS ds__week
@@ -18,6 +20,11 @@ SELECT
   , DATE_TRUNC(ds_partitioned, month) AS ds_partitioned__month
   , DATE_TRUNC(ds_partitioned, quarter) AS ds_partitioned__quarter
   , DATE_TRUNC(ds_partitioned, isoyear) AS ds_partitioned__year
+  , booking_paid_at
+  , DATE_TRUNC(booking_paid_at, isoweek) AS booking_paid_at__week
+  , DATE_TRUNC(booking_paid_at, month) AS booking_paid_at__month
+  , DATE_TRUNC(booking_paid_at, quarter) AS booking_paid_at__quarter
+  , DATE_TRUNC(booking_paid_at, isoyear) AS booking_paid_at__year
   , is_instant AS create_a_cycle_in_the_join_graph__is_instant
   , ds AS create_a_cycle_in_the_join_graph__ds
   , DATE_TRUNC(ds, isoweek) AS create_a_cycle_in_the_join_graph__ds__week
@@ -29,6 +36,11 @@ SELECT
   , DATE_TRUNC(ds_partitioned, month) AS create_a_cycle_in_the_join_graph__ds_partitioned__month
   , DATE_TRUNC(ds_partitioned, quarter) AS create_a_cycle_in_the_join_graph__ds_partitioned__quarter
   , DATE_TRUNC(ds_partitioned, isoyear) AS create_a_cycle_in_the_join_graph__ds_partitioned__year
+  , booking_paid_at AS create_a_cycle_in_the_join_graph__booking_paid_at
+  , DATE_TRUNC(booking_paid_at, isoweek) AS create_a_cycle_in_the_join_graph__booking_paid_at__week
+  , DATE_TRUNC(booking_paid_at, month) AS create_a_cycle_in_the_join_graph__booking_paid_at__month
+  , DATE_TRUNC(booking_paid_at, quarter) AS create_a_cycle_in_the_join_graph__booking_paid_at__quarter
+  , DATE_TRUNC(booking_paid_at, isoyear) AS create_a_cycle_in_the_join_graph__booking_paid_at__year
   , listing_id AS listing
   , guest_id AS guest
   , host_id AS host
@@ -39,4 +51,4 @@ SELECT
 FROM (
   -- User Defined SQL Query
   SELECT * FROM ***************************.fct_bookings
-) bookings_source_src_10000
+) bookings_source_src_10001
